;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;

	include hv.inc
	
	CONFIG	OSC=INTIO2
	CONFIG	FCMEN=ON
	CONFIG	PWRT=ON
	CONFIG	BOR=OFF
	CONFIG	BORV=3
	CONFIG	WDT=OFF
	CONFIG	PBADEN=DIG
	CONFIG	CCP2MX=RB3
	CONFIG	STVREN=ON
	CONFIG	LVP=OFF
	
	
var_g	UDATA_ACS	0x0

sfsr	res	.6	; saved SFR, at low level interrupt
hfsr	res	.2	; saved SFR, at high level interrupt
t0fsr	res	.2	; timer0's FSR0
i_regs	res	.3	; interrupt context
GFlag	res	.1	; generic flags
BFlag	res	.1	; button flags
TFlag	res	.1	; time flags
PFlag	res	.1	; power flags
CFlag	res	.1	; clock options flags (24h, leading zero suppress,...)
Vout	res	.1
lat_t	res	.1
d1	res	.1
d2	res	.1
d3	res	.1
d4	res	.1
rand	res	.2	; random generator


	GLOBAL	sfsr, hfsr, t0fsr, Vout, lat_t, d1, d2, d3, rand
	GLOBAL	GFlag, BFlag, PFlag, TFlag, CFlag

var_b0	UDATA	0x80

pair0		res	.8
pair1		res	.8
tval		res	.4
tcount		res	.2
tcurrent	res	.1
tmin		res	.2
ttime		res	.1
ttotal		res	.1
tloop		res	.2
tpair		res	.1
tdigit		res	.1
max_bright	res	.2

value		res	.1
day_ptr		res	.4
hour_ptr	res	.4
minute_ptr	res	.4
qsec_ptr	res	.4
tube_mask	res	.3
acount		res	.1
cdown		res	.1
rand_tmp	res	.1
saved_tblptr	res	.2

but0_tick	res	.1
but1_tick	res	.1
but0_ignore	res	.1
but1_ignore	res	.1
timeout		res	.1

tick		res	.1
time		res	.14	; SS MM HH dow DD MM YY  x2 (BCD)
qsec		res	.1

drift		res	.4	; number of 64th second ticks before adjusting
drift_value	res	.1	; adjustment (signed value +1, 0 or -1)

date		res	.3	; DD MM YY (used in time.asm)

ed_values	res	.4
ed_mode		res	.1
ed_tube		res	.1
ed_max		res	.1
ed_tmp		res	.1

t_tmp		res	.2
dow		res	.2

fade_tmp	res	.2
	

	GLOBAL	pair0, pair1, tval, tcount, tcurrent, tmin, ttime
	GLOBAL	ttotal, tloop, tpair, tdigit, but0_tick, but1_tick
	GLOBAL	value, day_ptr, hour_ptr, minute_ptr, qsec_ptr
	GLOBAL	tube_mask, acount, tick, time, qsec, timeout
	GLOBAL	but0_ignore, but1_ignore, rand_tmp, max_bright
	GLOBAL	ed_values, ed_mode, ed_tube, ed_max, ed_tmp
	GLOBAL	date, t_tmp, dow, saved_tblptr
	GLOBAL	drift, drift_value, fade_tmp
	
var_b1	UDATA	0x100

tlist		res	.24
tchange		res	.10

auto_fading	res	NUM_TUBES
auto_brightness	res	NUM_TUBES
auto_values	res	NUM_TUBES*2
auto_flags	res	NUM_TUBES

	GLOBAL	tlist, tchange, auto_fading, auto_values
	GLOBAL	auto_brightness, auto_flags
	
	
; hv.asm
	extern	init_power, check_power, int_ad, int_power
; delay.asm
	extern	delay_1ms, delay_10ms, delay_100ms, delay_1s
; nixie.asm
	extern	int_timer0, tube_init, rebuild_tlist, set_data
; clock.asm
	extern	int_timer1, timer1_work
; math.asm
	extern	init_random
; tables.asm
	extern	remap_digit
; editor.asm
	extern	check_buttons, set_dow

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Entry vectors
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
main_entry	CODE 0x00
	goto	start
	
int0_entry	CODE 0x8
	goto	int_hi_handler
	
int1_entry	CODE 0x18
	goto	int_lo_handler


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Interrupt handler
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
int_code	CODE

int_hi_handler
	btfsc	PIR1,ADIF		; AD converter => control high-voltage
	call	int_ad
	;
	btfsc	INTCON,INT0IF		; INT0 => power lost
	call	int_power
	;
	btfsc	PIR1,TMR1IF		; TIMER 1 => real-time clock
	call	int_timer1
	;
	retfie FAST
	
int_lo_handler
	movwf	i_regs
	movff	STATUS,i_regs+1
	movff	BSR,i_regs+2
	;
	btfsc	INTCON,TMR0IF		; TIMER 0 => refresh tube
	call	int_timer0
	;
	movff	i_regs+2,BSR
	movf	i_regs,w
	movff	i_regs+1,STATUS
	retfie
	
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
main_code	CODE

start
	call	init_ports
	call	init_data
	call	init_random
	call	tube_init
	bsf	INTCON,GIEH
	bsf	INTCON,GIEL
	call	init_power

main_loop
	call	timer1_work
	call	check_power
	call	check_buttons
	bra	main_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;
; Initialize RAM.
; - bzero everything
; - initialize the required data
;
init_data
	clrf	GFlag			; CLEAR ALL BANKS
	clrf	WREG
	lfsr	FSR0,0
id0	clrf	POSTINC0		; clear bank0
	decfsz	WREG,f
	bra	id0
id1	clrf	POSTINC0		; clear bank1
	decfsz	WREG,f
	bra	id1
	nop				; INIT DATA
	
	movlw	0x8			; initial maximum brightness
	movwf	max_bright
	
	movlw	0x00			; drift +1 second per day 
	movwf	drift+1
	movlw	0xA8
	movwf	drift+0
	movlw	-1
	movwf	drift_value
	incf	drift+2,f		; drift counter = 0x0001
		
	movlw	0x00
	movwf	t_sec
	incf	WREG,w
	movwf	t2_sec			; time + 1 second
	movlw	0x23			; HH
	movwf	t_hour
	movwf	t2_hour
	movlw	0x58			; MM
	movwf	t_min
	movwf	t2_min
	movlw	0x01			; DD
	movwf	t_day
	movwf	t2_day
	movlw	0x12			; MM
	movwf	t_month
	movwf	t2_month
	movlw	0x10			; YY
	movwf	t_year
	movwf	t2_year
	call	set_dow			; compute day of week
	return


;
; Setup ports, interrupts, etc...
;
init_ports
					; OSCILLATOR
	movf	OSCCON,w
	iorlw	0x70
	movwf	OSCCON
	bsf	OSCTUNE,PLLEN		; enable PLL, to get 8MHz x 4
	
	bsf	RCON,IPEN		; enable interrupt priority
	
					; TIMER 0
	movlw	0x46			; 8 bits, prescale 1:128, OFF
	movwf	T0CON
	bsf	INTCON,TMR0IE
	bcf	INTCON2,TMR0IP
					; TIMER 1
	movlw	0x8F			; oscillator on, async, external clock
#if 0
	movlw	0xBF			; 8x slower to debug fading
#endif
	movwf	T1CON			; 16 bits, prescale 1:1
	bsf	PIE1,TMR1IE
	bsf	IPR1,TMR1IP		; high priority
	
					; INT 0 (always high priority)
	bsf	INTCON2,INTEDG0		; positive edge
	bsf	INTCON,INT0IE
	
init_tris
	GLOBAL	init_tris
	bcf	ANODE0			; Nixie anode drivers
	bcf	ANODE1
	bcf	ANODE0_DIR
	bcf	ANODE1_DIR

	movf	TRISB,w			; 8 bits output (to Nixie drivers)
	andlw	0x0F
	movwf	TRISB
	movf	TRISC,w
	andlw	0x0F
	movwf	TRISC

	bsf	POWER_DIR		; Power detect

	bsf	BUTTON0_DIR		; Buttons
	bsf	BUTTON1_DIR
	
	return





	END
		