;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;
	
;
; Time & Date routines
;
	include hv.inc
	
	; variables
	extern	time, TFlag, GFlag, t_tmp, date, dow, hfsr
	
	; functions
	extern	days_per_month, dow_months_table, dow_months_table_leap
	

time_code CODE

;
; One second has elapsed. 
;
; Called from int_timer1. Must save and restore FSR1.
;
incr_time
	GLOBAL	incr_time

	movff	FSR1L,hfsr+0		; save FSR1
	movff	FSR1H,hfsr+1
	
	lfsr	FSR1,t_sec		; update time and
	bsf	GFlag,UPDATE_TFLAG	; flag updates
	clrf	TFlag
	call	increment_time

	lfsr	FSR1,t2_sec		; update time+1second
	bcf	GFlag,UPDATE_TFLAG	; don't flag updates
	call	increment_time

	movff	hfsr+0,FSR1L		; restore FSR1
	movff	hfsr+1,FSR1H

	return
	

;
; increment time (time pointed by FSR1) : 1 second has elapsed
;
; FSR1: seconds minutes hours day_of_week (0-6) day (1-31) month year
;
; if UPDATE_TFLAG is set, then update time flags.
;	
increment_time
	;; SECOND
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_SEC
	incf	INDF1,w			; seconds++
	daw
	movwf	INDF1
	movlw	0x60			; if seconds != 60
	cpfseq	INDF1			; then return
	return
	;; MINUTE
	clrf	POSTINC1		; seconds = 0
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_MIN
	incf	INDF1,w			; minutes++
	daw
	movwf	INDF1
	movlw	0x60			; if minutes != 60
	cpfseq	INDF1			; then return
	return
	;; HOUR
	clrf	POSTINC1		; minutes = 0
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_HOUR
	incf	INDF1,w			; hours++
	daw
	movwf	INDF1
	movlw	0x24			; if hours != 24
	cpfseq	INDF1			; then return
	return
	;; DAY OF WEEK
	clrf	POSTINC1		; hours = 0
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_DAY
	incf	INDF1,w			; day_of_week++
	movlw	0x7			; if day_of_week == 7
	cpfseq	INDF1
	bra	inct_day
	clrf	INDF1			; then day_of_week = 0
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_WEEK
inct_day
	;; DAY
	movlw	0x2			; FSR1 points to dow
	movff	PLUSW1,date+1		; so month is at offset +2
	movlw	0x3			; and year is at offset +3
	movff	PLUSW1,date+2
	call	get_days		; get number of days in this month
	cpfseq	PREINC1			; are we on the last day ?
	return				; no... done.
	;; MONTH
	movlw	0x01			; yes... reset day to "01"
	movwf	POSTINC1
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_MONTH
	incf	INDF1,w			; month++
	daw
	movwf	INDF1
	movlw	0x13			; if month != 13
	cpfseq	INDF1			; then return
	return
	;; YEAR
	movlw	0x1			; reset month to "01"
	movwf	POSTINC1
	btfsc	GFlag,UPDATE_TFLAG
	bsf	TFlag,E_YEAR
	incf	INDF1,w			; year++
	daw
	movwf	INDF1
	;; no check for year > 99, this poor clock will be dead by 2099.
	return
	

;
; Convert 24 hours -> 12 hours
;
; hours_24 in W (BCD). returns hours_12 in W (BCD)
;
; see http://en.wikipedia.org/wiki/12-hour_clock
;
hours24_to_12
	GLOBAL	hours24_to_12
	addlw	0xEE			; test if smaller than 0x12
	btfss	STATUS,C
	bra	h24_01
	addlw	0x12			; undo +0xEE
	addlw	0x88			; add 9's complement
	daw				; BCD adjust
	addlw	0x00			; update Z flag (daw doesn't do it)
h24_00	btfsc	STATUS,Z		; if (WREG == 0)
	movlw	0x12			; then return 0x12
	return				; else return WREG
h24_01	addlw	0x12			; undo +0xEE
	bra	h24_00	
	
	
;
; Test W (in BCD) for leap year. Year is 2000+W
;
; Algorithm:
;   Y mod 400 == 0  => LEAP
;   Y mod 100 == 0  => NOT LEAP
;   Y mod   4 == 0  => LEAP
;
; Return: Carry set if leap.
;
is_leap_year
	bz	leap			; 2000 is leap
	movwf	t_tmp
	UNBCD	t_tmp
	sublw	.100
	bz	not_leap		; 2100 is not leap
	sublw	.100
	bz	not_leap		; 2200 is not leap
	movf	t_tmp,w
	andlw	0x3
	bnz	not_leap		; if mod 4 != 0 then not leap
leap
	bsf	STATUS,C
	return
not_leap
	bcf	STATUS,C
	return	
	
	
;
; Get number of days for current month (DD/MM/YY in 'date')
; (result returned in W, in BCD !)
;
get_days_bcd
	GLOBAL	get_days_bcd
	movf	date+1,w
	call	days_per_month	
	movwf	t_tmp+1
	movlw	.2
	cpfseq	date+1			; february ?
	bra	gd_10			; no. return value from table
	movf	date+2,w		; yes. check for leap year
	call	is_leap_year
	btfsc	STATUS,C		; if leap year,
	incf	t_tmp+1,f		; then 29 days.
gd_10	movf	t_tmp+1,w
	return
	
	
;
; Same as above, but result returned in binary
;
get_days
	GLOBAL	get_days
	call	get_days_bcd
	UNBCD	t_tmp+1
	return
	
	
;
; Compute day of week for a given 'date' (DD/MM/20YY)
; http://en.wikipedia.org/wiki/Calculating_the_day_of_the_week
;
; 0 = Sunday
; 1 = Monday
; ...
; 6 = Saturday
;
get_dow
	GLOBAL	get_dow
	movlw	.6			; century 20YY is 6
	movwf	dow
	movff	date+2,dow+1		; get YY
	UNBCD	dow+1			; convert to binary
	addwf	dow,f			; and add to 'dow'
	rrncf	dow+1,f			; divide YY by 4
	rrncf	dow+1,w
	andlw	0x3F
	addwf	dow,f			; and add to 'dow'
	call	months_table		; lookup month table
	addwf	dow,f			; add result to 'dow'
	movff	date,dow+1		; get DD
	UNBCD	dow+1			; convert to binary
	addwf	dow,f			; and add to 'dow'

	movlw	.6			; compute remainder of dow / 7
gdow_00	cpfsgt	dow
	bra	gdow_10
	subwf	dow,f
	decf	dow,f
	bra	gdow_00
gdow_10	movf	dow,w			; dow is the day of week.
	return	



	
; lookup months table	
months_table
	movf	date+2,w
	call	is_leap_year		; is 'date' a leap year ?
	movf	date+1,w
	btfss	STATUS,C
	goto	dow_months_table
	goto	dow_months_table_leap
	
	
	END
	